@@ -219,6 +219,7 @@ static gboolean g_idle_dispatch    (GSource     *source,
 
 G_LOCK_DEFINE_STATIC (main_loop);
 static GMainContext *default_main_context;
+static GSList *main_contexts_without_pipe = NULL;
 
 #if defined(G_PLATFORM_WIN32) && defined(__GNUC__)
 __declspec(dllexport)
@@ -267,17 +268,14 @@ g_poll (GPollFD *fds,
   for (f = fds; f < &fds[nfds]; ++f)
     if (f->fd >= 0)
       {
-	if (f->events & G_IO_IN)
+	if (f->fd == G_WIN32_MSG_HANDLE)
+	  poll_msgs = TRUE;
+	else
 	  {
-	    if (f->fd == G_WIN32_MSG_HANDLE)
-	      poll_msgs = TRUE;
-	    else
-	      {
 #ifdef G_MAIN_POLL_DEBUG
-		g_print ("g_poll: waiting for %#x\n", f->fd);
+	    g_print ("g_poll: waiting for %#x\n", f->fd);
 #endif
-		handles[nhandles++] = (HANDLE) f->fd;
-	      }
+	    handles[nhandles++] = (HANDLE) f->fd;
 	  }
       }
 
@@ -326,7 +324,10 @@ g_poll (GPollFD *fds,
 		   */
 		  timer = SetTimer (NULL, 0, timeout, NULL);
 		  if (timer == 0)
-		    g_warning (G_STRLOC ": SetTimer() failed");
+		    {
+		      g_warning (G_STRLOC ": SetTimer() failed");
+		      ready = WAIT_TIMEOUT;
+		    }
 		  else
 		    {
 #ifdef G_MAIN_POLL_DEBUG
@@ -380,7 +381,7 @@ g_poll (GPollFD *fds,
     }
 
 #ifdef G_MAIN_POLL_DEBUG
-  g_print ("wait returns %d%s\n",
+  g_print ("wait returns %ld%s\n",
 	   ready,
 	   (ready == WAIT_FAILED ? " (WAIT_FAILED)" :
 	    (ready == WAIT_TIMEOUT ? " (WAIT_TIMEOUT)" :
@@ -407,10 +408,13 @@ g_poll (GPollFD *fds,
   else if (ready >= WAIT_OBJECT_0 && ready < WAIT_OBJECT_0 + nhandles)
     for (f = fds; f < &fds[nfds]; ++f)
       {
-	if ((f->events & G_IO_IN)
+	if ((f->events & (G_IO_IN | G_IO_OUT))
 	    && f->fd == (gint) handles[ready - WAIT_OBJECT_0])
 	  {
-	    f->revents |= G_IO_IN;
+	    if (f->events & G_IO_IN)
+	      f->revents |= G_IO_IN;
+	    else
+	      f->revents |= G_IO_OUT;
 #ifdef G_MAIN_POLL_DEBUG
 	    g_print ("g_poll: got event %#x\n", f->fd);
 #endif
@@ -568,6 +572,9 @@ g_main_context_unref_and_unlock (GMainContext *context)
       CloseHandle (context->wake_up_semaphore);
 #endif
     }
+  else
+    main_contexts_without_pipe = g_slist_remove (main_contexts_without_pipe, 
+						 context);
 #endif
   
   g_free (context);
@@ -590,6 +597,45 @@ g_main_context_unref (GMainContext *context)
   g_main_context_unref_and_unlock (context);
 }
 
+#ifdef G_THREADS_ENABLED
+static void 
+g_main_context_init_pipe (GMainContext *context)
+{
+# ifndef G_OS_WIN32
+  if (pipe (context->wake_up_pipe) < 0)
+    g_error ("Cannot create pipe main loop wake-up: %s\n",
+	     g_strerror (errno));
+  
+  context->wake_up_rec.fd = context->wake_up_pipe[0];
+  context->wake_up_rec.events = G_IO_IN;
+# else
+  context->wake_up_semaphore = CreateSemaphore (NULL, 0, 100, NULL);
+  if (context->wake_up_semaphore == NULL)
+    g_error ("Cannot create wake-up semaphore: %s",
+	     g_win32_error_message (GetLastError ()));
+  context->wake_up_rec.fd = (gint) context->wake_up_semaphore;
+  context->wake_up_rec.events = G_IO_IN;
+#  ifdef G_MAIN_POLL_DEBUG
+  g_print ("wake-up semaphore: %#x\n", (guint) context->wake_up_semaphore);
+#  endif
+# endif
+  g_main_context_add_poll_unlocked (context, 0, &context->wake_up_rec);
+}
+
+void
+g_main_thread_init ()
+{
+  GSList *curr = main_contexts_without_pipe;
+  while (curr)
+    {
+      g_main_context_init_pipe ((GMainContext *)curr->data);
+      curr = curr->next;
+    }
+  g_slist_free (main_contexts_without_pipe);
+  main_contexts_without_pipe = NULL;  
+}
+#endif /* G_THREADS_ENABLED */
+
 /**
  * g_main_context_new:
  * 
@@ -630,28 +676,10 @@ g_main_context_new ()
 
 #ifdef G_THREADS_ENABLED
       if (g_thread_supported ())
-	{
-#ifndef G_OS_WIN32
-	  if (pipe (context->wake_up_pipe) < 0)
-	    g_error ("Cannot create pipe main loop wake-up: %s\n",
-		     g_strerror (errno));
-	  
-	  context->wake_up_rec.fd = context->wake_up_pipe[0];
-	  context->wake_up_rec.events = G_IO_IN;
-	  g_main_context_add_poll_unlocked (context, 0, &context->wake_up_rec);
-#else
-	  context->wake_up_semaphore = CreateSemaphore (NULL, 0, 100, NULL);
-	  if (context->wake_up_semaphore == NULL)
-	    g_error ("Cannot create wake-up semaphore: %s",
-		     g_win32_error_message (GetLastError ()));
-	  context->wake_up_rec.fd = (gint) context->wake_up_semaphore;
-	  context->wake_up_rec.events = G_IO_IN;
-#ifdef G_MAIN_POLL_DEBUG
-	  g_print ("wake-up semaphore: %#x\n", (guint) context->wake_up_semaphore);
-#endif
-	  g_main_context_add_poll_unlocked (context, 0, &context->wake_up_rec);
-#endif
-	}
+    g_main_context_init_pipe (context);
+  else
+    main_contexts_without_pipe = g_slist_prepend (main_contexts_without_pipe, 
+						  context);
 #endif
 
   return context;
