@@ -289,7 +289,9 @@ g_scanner_msg_handler (GScanner		*scanner,
 {
   g_return_if_fail (scanner != NULL);
   
-  fprintf (stderr, "%s:%d: ", scanner->input_name, scanner->line);
+  fprintf (stderr, "%s:%d: ",
+	   scanner->input_name ? scanner->input_name : "<memory>",
+	   scanner->line);
   if (is_error)
     fprintf (stderr, "error: ");
   fprintf (stderr, "%s\n", message);
@@ -954,7 +956,7 @@ g_scanner_unexp_token (GScanner		*scanner,
     case G_TOKEN_OCTAL:
     case G_TOKEN_INT:
     case G_TOKEN_HEX:
-      g_snprintf (token_string, token_string_len, "number `%ld'", scanner->value.v_int);
+      g_snprintf (token_string, token_string_len, "number `%lu'", scanner->value.v_int);
       break;
       
     case G_TOKEN_FLOAT:
@@ -1558,15 +1560,15 @@ g_scanner_get_token_ll	(GScanner	*scanner,
 	  switch (token)
 	    {
 	    case G_TOKEN_BINARY:
-	      value.v_binary = strtol (gstring->str, &endptr, 2);
+	      value.v_binary = strtoul (gstring->str, &endptr, 2);
 	      break;
 	      
 	    case G_TOKEN_OCTAL:
-	      value.v_octal = strtol (gstring->str, &endptr, 8);
+	      value.v_octal = strtoul (gstring->str, &endptr, 8);
 	      break;
 	      
 	    case G_TOKEN_INT:
-	      value.v_int = strtol (gstring->str, &endptr, 10);
+	      value.v_int = strtoul (gstring->str, &endptr, 10);
 	      break;
 	      
 	    case G_TOKEN_FLOAT:
@@ -1574,7 +1576,7 @@ g_scanner_get_token_ll	(GScanner	*scanner,
 	      break;
 	      
 	    case G_TOKEN_HEX:
-	      value.v_hex = strtol (gstring->str, &endptr, 16);
+	      value.v_hex = strtoul (gstring->str, &endptr, 16);
 	      break;
 	      
 	    default:
